#!/usr/bin/env python3

import sys, re, csv, decimal
import time 
from lib.utils import *
from lib.paraver import *
from lib.events import *
from lib.genericCSVtoPRV import *
from lib.asm2prv import *
from lib.ParaverLine import *


def getValuesHashedFromDisasm(disasm, paraverEvents, paraverLine, start, end, add_new=False):
    position = start
    args = getInstructionArguments(disasm)

    for a in args:
        a = a.replace(" ", "")
        a = a.replace("(", "")
        a = a.replace(")", "")

        if not a in paraverEvents[position].values:

            if not add_new:
                continue

            num_values = len(paraverEvents[position].values)
            paraverEvents[position].addValue(num_values + 1, a)

        paraverLine.addEvent(paraverEvents[position], paraverEvents[position].values[a])
        position = position + 1

    while position <= end:
        paraverLine.addEvent(paraverEvents[position], 0)
        position = position + 1


def getImmediateFromDisasm(disasm, paraverEvents, paraverLine):
    match = re.search(" (0x)?([0-9]+)(,|\(|$)", disasm)
    if match is not None:
        imm = match.group(0)
        imm = imm.replace(" ", "")
        paraverLine.addEvent(paraverEvents[4], imm)

def getAddrFromDisasm(disasm, paraverevents: PrvEvent, paraverline: ParaverLine, raddr_index, imm_index):
    match = re.search(" ((0x)?[0-9]*)?\((.*)\)", disasm)
    if match is not None:
        imm = match.groups()[0]
        reg = match.groups()[2]
        if imm == "":
            imm = 0
        else:
            imm = int(imm,0)
        paraverline.addEvent(paraverevents[raddr_index], paraverevents[raddr_index].values[reg])
        paraverline.addEvent(paraverevents[imm_index], imm)

        #print("Pillando direcciones")


def disasmToString(disasm, paraverEvents, paraverLine, last_state=[False, False, False]):
    #Constants
    insts_without_rdest = ["vse"]
    insts_with_addr = ["vse", "vle"]
    insts_setvec = ["vsetli"]

    rdest_index   = [0]
    rsrc_index    = [1,2,3]
    imm_index     = [4,5]
    vs_index      = [6,7,8]
    inst_index    = [9]
    raddr_index   = [10]
    immaddr_index = [11]

    last_inst              = None
    last_inst_witout_rdest = last_state[0]
    last_inst_with_addr    = last_state[1]
    last_inst_setvec       = last_state[2]

    current_inst_with_addr    = False
    current_inst_witout_rdest = False
    current_inst_setvec       = False



    inst = getInstruction(disasm)
    paraverLine.addEvent(paraverEvents[inst_index[0]], paraverEvents[inst_index[0]].values[inst])

    if inst in insts_setvec:

        getValuesHashedFromDisasm(disasm, paraverEvents, paraverLine, vs_index[0], vs_index[-1], False)

        last_inst_setvec    = False
        current_inst_setvec = True

    if inst in insts_with_addr:

        getAddrFromDisasm(disasm, paraverEvents, paraverLine, raddr_index[0], immaddr_index[0])

        last_inst_with_addr    = False
        current_inst_with_addr = True

    if inst in insts_without_rdest:
        getValuesHashedFromDisasm(disasm, paraverEvents, paraverLine, rsrc_index[0], rsrc_index[-1], False)
        paraverLine.addEvent(paraverEvents[rdest_index[0]], 0)
        last_inst_witout_rdest    = False
        current_inst_witout_rdest = True

    else:
        getValuesHashedFromDisasm(disasm, paraverEvents, paraverLine, rdest_index[0], rsrc_index[1], False)

    getImmediateFromDisasm(disasm, paraverEvents, paraverLine)

    if last_inst_setvec:
        paraverLine.addEvent(paraverEvents[vs_index[0]], 0)
        paraverLine.addEvent(paraverEvents[vs_index[1]], 0)
        paraverLine.addEvent(paraverEvents[vs_index[2]], 0)

    if last_inst_with_addr:
        paraverLine.addEvent(paraverEvents[raddr_index[0]], 0)
        paraverLine.addEvent(paraverEvents[immaddr_index[0]], 0)

    #if last_inst_witout_rdest:
    #    paraverLine.addEvent(paraverEvents[rdest_index[0]], 0)

    getImmediateFromDisasm(disasm, paraverEvents, paraverLine)

    last_inst_witout_rdest = current_inst_witout_rdest
    last_inst_with_addr = current_inst_with_addr
    last_inst_setvec = current_inst_setvec
    last_state = [last_inst_witout_rdest, last_inst_with_addr,last_inst_setvec]

    return last_state

def offset_parser(offsets_string, paraver_events, address, paraver_line):
    offsets = offsets_string.split(',')
    events = paraver_events.derivedEvents

    counter = 0
    for offset in offsets:
        value = address + int(offset, 0)
        paraver_line.addEvent(events[counter], value)
        counter = counter + 1

def user_events(row, paraverEvents, paraver_line):
    assert row.startswith(".trace")

    row = row[len(".trace"):];

    rdest_index   = [0]

    row_split = row.split(",")
    usr_event = PrvEvent("",int(row_split[0]))

    paraver_line.addEvent(paraverEvents[rdest_index[0]], 0)
    paraver_line.addEvent(usr_event, int(row_split[1]))

def getNumberOfThreadsFromTrace(csvfile):

    data = csv.reader(csvfile, skipinitialspace=True, delimiter=',')

    line = next(data)

    if line[-1] != 'core-id':
        return 1
    
    num_cores = 0
    num_threads = 0
    for row in data:
        if int(row[-1]) > num_cores:
            num_cores = int(row[-1])
        if int(row[-2]) > num_threads:
            num_threads = int(row[-2])            

    return num_threads + 1

def vehaveTraceToPrv(csvfile, prvfile, PrvEvents, nThreads):

    #check if the trace file includes information about nunber of cores and threads        
    # data = csv.reader(csvfile, skipinitialspace=True, delimiter=',')

    # line = next(data)

    # if line[-1] == 'core-id':
    #     num_cores = getNumberOfCoresFromTrace(data)

    # print("Number of cores=" + str(num_cores))
    
    # csvfile.seek(0)
    header = paraverHeader(csvfile, nthreads=nThreads, delimiter=",")
    writePRVFile(prvfile, header)

    csvfile.seek(0)
    data = csv.reader(csvfile, skipinitialspace=True, delimiter=',')
    
    paraver_line = ParaverLine(2,1,1,1,1)

    last_dst_value = 0
    last_dst_event = PrvEvents[2].derivedEvents[0]

    last_user_event = 0
    last_state = [False, False, False]
    for row in data:

        if(sanityCheck(row, True) == -1): continue

        paraver_line.time = row[0]
        paraver_line.events = []

        #set cpuId and threadId:
        if nThreads > 1:
            paraver_line.cpuId = int(row[-1]) + 1
            paraver_line.threadId = int(row[-2]) + 1                
        
        disasm_col=2
        address_col=5
        offset_col=8

        i=0
        address = 0 

        for col in row[1:]:
            i = i + 1
            val=None
            if i==disasm_col: #disasm
                if row[i][0] == '.':
                   user_events(row[i], PrvEvents[i].derivedEvents, paraver_line)	
                   last_state = [True, True, False]
                else:
                    last_state = disasmToString(row[i], PrvEvents[i].derivedEvents, paraver_line, last_state)

            elif col != "": 
               if i==address_col:
                   val = int(col, 0)
                   address = val
               elif i==offset_col:
                   offset_parser(col, PrvEvents[i], address, paraver_line)
               else:
                   val = int(col, 0)
            else :
                val = 0

            if val is not None:
                paraver_line.addEvent(PrvEvents[i], val)


        actual_dst_event = paraver_line.removeEvent(last_dst_event);
        paraver_line.addEvent(last_dst_event, last_dst_value)
        last_dst_value = actual_dst_event

        writePRVFile(prvfile, paraver_line.getLine())

    printMsg("PRV generated successfully!", "OK")

def generateOffsetEvents(events, num_events, start_ID):

    offset_event = PrvEvent("addres-VL", start_ID)

    counter = 0

    for i in range(start_ID+1, start_ID+num_events+1):
        name="addres-VL-"+str(counter)
        tmp_event = PrvEvent(name, i)
        counter = counter + 1
        offset_event.derivedEvents.append(tmp_event)

    events.append(offset_event)




def main():
    filename = sys.argv[1]

    start = time.time() 
    csvfile = openCSVFile(filename)
    prvfile = openPRVFile(filename, compress=False)

    eventsNames = readGenericCSVHeader(csvfile,",")

    prvEvents = getStdEventsFromJsonFile(eventsNames, "vehave")

    scriptpath = os.path.dirname(os.path.realpath(__file__))

    getRegistersValuesFromJson(scriptpath + "/lib/RISCV_ISA.json", prvEvents[2].derivedEvents[0])
    getRegistersValuesFromJson(scriptpath + "/lib/RISCV_ISA.json", prvEvents[2].derivedEvents[1])
    getRegistersValuesFromJson(scriptpath + "/lib/RISCV_ISA.json", prvEvents[2].derivedEvents[2])
    getRegistersValuesFromJson(scriptpath + "/lib/RISCV_ISA.json", prvEvents[2].derivedEvents[3])
    getRegistersValuesFromJson(scriptpath + "/lib/RISCV_ISA.json", prvEvents[2].derivedEvents[10])
    assignValuesFromJson(scriptpath + "/lib/RISCV_ISA.json", "Vtype",  prvEvents[2].derivedEvents[6])
    assignValuesFromJson(scriptpath + "/lib/RISCV_ISA.json", "Vtype",  prvEvents[2].derivedEvents[7])
    assignValuesFromJson(scriptpath + "/lib/RISCV_ISA.json", "Instructions",  prvEvents[2].derivedEvents[9])

    generateOffsetEvents(prvEvents, 2048, 48000000)

    nthreads = getNumberOfThreadsFromTrace(csvfile)
    printMsg("Number of threads in trace is {}".format(nthreads), "INFO")
    
    end = time.time() 

    pre_time = round(end-start, 2)
    printMsg("Preprocessing time: " + str(pre_time)+ " seconds", "INFO") 

    start = time.time()

    vehaveTraceToPrv(csvfile, prvfile, prvEvents, nthreads)
    #createRow(filename, "vehave")
    createRow(filename, "vehave", nthreads)    
    createPcf(filename, prvEvents)
    end = time.time() 

    post_time = round(end-start, 2)
    printMsg("Processing time: " + str(post_time)+ " seconds", "INFO") 

if __name__ == "__main__":
    main()
